﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

using View;
using System.Diagnostics;

namespace Model
{
    /// <summary>
    /// The Mage model class, extends Unit
    /// \n This Unit has strong magical capabilities, and is capable of powerful ranged magic attacks, but makes up with poor physical stats
    /// </summary>
    class Mage : Unit
    {
        /**
        Sets and returns whether or not unit is alive
        */
        public bool Alive { get; set; }
        /**
        Sets and returns a unit's HP
        */
        public int Hp { get; set; }
        private int str;                // unit strength
        private int intelligence;       // unit intelliegence
        private int skill;              // unit skill
        /**
        Sets and returns a unit's Speed
        */
        public int Speed { get; set; }
        /**
        Sets and returns a unit's Defense
        */
        public int Def { get; set; }
        /**
        Sets and returns a unit's Resistance
        */
        public int Res { get; set; }
        /**
        Sets and returns a unit's Level
        */
        public int Level { get; set; }
        private readonly int movability = 4;        // Warrior movement is permanently set to 4
        private Weapon[] equipableWeapons;          // Array of all equipable weapons
        /**
        returns weapon the unit is currently equipping
        */
        public Weapon equippedWeapon { get; set; }
        private Texture2D spriteImage;              // the char sprite
        private Vector2 pixelCoordinates;           // the pixel coordinate of the unit
        private Tuple<int, int> position;           // the tile location of the unit
        private Button[] buttons = new Button[5];   // the buttons associated with the unit
        private Texture2D charInfo, charAttackInfo; // The character and attack info textures
        private int currentFrame;                   // the current frame the sprite is on
        private int player;                         // which player this unit belongs to

        /**
        * The constructor for Unit Mage
        * @param spriteImage The character sprite
        * @param attackButton The dropdownmenu attack button texture
        * @param moveButton The dropdownmenu move button texture
        * @param itemButton The dropdownmenu item button texture
        * @param waitButton The dropdownmenu wait button texture
        * @param confirmButton The attack confirm button texture
        * @param charInfo The character info popup texture
        * @param charAttackInfo The character attack menu popup texture
        * @param coordinates The unit's current coordinate on screen
        * @param player The player of which the unit belongs to
        */
        public Mage(Texture2D spriteImage, Button attackButton, Button moveButton,
            Button itemButton, Button waitButton, Button confirmButton, Texture2D charInfo,
            Texture2D charAttackInfo, Vector2 coordinates, int player)
        {
            this.spriteImage = spriteImage;
            buttons[0] = attackButton;
            buttons[1] = moveButton;
            buttons[2] = itemButton;
            buttons[3] = waitButton;
            buttons[4] = confirmButton;
            this.charInfo = charInfo;
            this.charAttackInfo = charAttackInfo;
            this.player = player;
            pixelCoordinates = coordinates;
            int positionX = (int)Math.Round(coordinates.X / 32);
            int positionY = (int)Math.Round(coordinates.Y / 32);
            position = new Tuple<int, int>(positionX, positionY);
            currentFrame = 1;
            setButtonCoordinates(pixelCoordinates);
            setInitialStats(); //sets initial warrior stats
        }

        //sets initial unit stats
        public void setInitialStats()
        {
            Alive = true;
            Level = 1;
            Hp = 8;
            Str = 2;
            Int = 7;
            Skill = 4;
            Speed = 8;
            Def = 3;
            Res = 8;
        }

        /*
        Sets the new strength value
        \n Gets the effective strength -> Unit strength + weapon strength
        */
        public int Str
        {
            get
            {
                return str + equippedWeapon.modStr;
            }
            set
            {
                str = value;
            }
        }

        /*
        Sets the new intelligence value
        \n Gets the effective intelligence -> Unit intelligence + weapon intelligence
        */
        public int Int
        {
            get
            {
                return intelligence + equippedWeapon.modInt;
            }
            set
            {
                intelligence = value;
            }
        }

        /*
        Sets the new skill value
        \n Gets the effective skill -> Unit skill + weapon skill
        */
        public int Skill
        {
            get
            {
                return skill + equippedWeapon.modSkill;
            }
            set
            {
                skill = value;
            }
        }

        /**
        Returns the unit's movability range on grid (number of spaces the unit can move in one turn)
        */
        public int getMovability()
        {
            return movability;
        }

        /**
        returns all stats as an array
        */
        public int[] getStats()
        {
            int[] allStats = new int[7];
            allStats[0] = Level;
            allStats[1] = Str;
            allStats[2] = Int;
            allStats[3] = Skill;
            allStats[4] = Speed;
            allStats[5] = Def;
            allStats[6] = Res;
            return allStats;
        }

        public Weapon[] getEquipableWeapons()
        {
            return equipableWeapons;
        }

        /**
        returns unit's class (warrior, mage, archer)
        */
        public UnitType getClass()
        {
            return UnitType.Warrior;
        }

        /**
        returns the sprite image of the unit
        */
        public Texture2D getSpriteImage()
        {
            return spriteImage;
        }

        /**
        returns the button texture at index i
        */
        public Texture2D getButtonImage(ButtonType buttonType)
        {
            switch (buttonType)
            {
                case ButtonType.Attack: // if attack clicked
                    return buttons[0].getImage();
                case ButtonType.Move: // if moved is clicked
                    return buttons[1].getImage();
                case ButtonType.Items: // if item is clicked
                    return buttons[2].getImage();
                case ButtonType.Wait: // if wait is clicked
                    return buttons[3].getImage();
                case ButtonType.AttackConfirm:
                    return buttons[4].getImage();
                default:
                    return null;
            }
        }

        /**
        indicates whether a button has already been previously selected or not
        */
        public bool isButtonActive(ButtonType buttonType)
        {
            switch (buttonType)
            {
                case ButtonType.Attack:
                    return buttons[0].Active;
                case ButtonType.Move:
                    return buttons[1].Active;
                case ButtonType.Items:
                    return buttons[2].Active;
                case ButtonType.Wait:
                    return buttons[3].Active;
                case ButtonType.AttackConfirm:
                    return buttons[4].Active;

                default:
                    return false;
            }
        }

        /**
        returns the char info screen texture
        */
        public Texture2D getCharInfo()
        {
            return charInfo;
        }

        /**
        returns the char attack info screen texture
        */
        public Texture2D getCharAttackInfo()
        {
            return charAttackInfo;
        }

        /**
        gets and sets unit's position by tile
        */
        public Tuple<int, int> Position
        {
            get
            {
                return position;
            }
            set
            {
                position = value;
                pixelCoordinates = new Vector2(value.Item1 * 32, value.Item2 * 32);
            }
        }

        /**
        returns the pixel coordinate of the unit
        \n sets the pixel coordinate, and also sets Position (which is the tile location of that coordinate)
        */
        public Vector2 PixelCoordinates
        {
            get
            {
                return pixelCoordinates;
            }
            set
            {
                pixelCoordinates = value;
                int positionX = (int)Math.Round(PixelCoordinates.X / 32);
                int positionY = (int)Math.Round(PixelCoordinates.Y / 32);
                position = new Tuple<int, int>(positionX, positionY);
            }
        }

        /**
        returns the dropdown menu buttons of the unit
        */
        public Button[] getButtons()
        {
            return buttons;
        }

        /**
        Returns the button type
        * @param buttonType The button to return (Move, Attack, Item, Wait, and attack confirm)
        */
        public Button getButtonOfType(ButtonType buttonType)
        {
            switch (buttonType)
            {
                case ButtonType.Attack:
                    return buttons[0];
                case ButtonType.Move:
                    return buttons[1];
                case ButtonType.Items:
                    return buttons[2];
                case ButtonType.Wait:
                    return buttons[3];
                case ButtonType.AttackConfirm:
                    return buttons[4];

                default:
                    return null;
            }
        }

        /**
        sets the coordinates of menu buttons
        * @param pixelCoordinates The pixel coordinate of the button
        */
        public void setButtonCoordinates(Vector2 pixelCoordinates)
        {
            int increment = 0;
            foreach (Button button in buttons)
            {
                if (button.Active)
                {
                    if (button.getButtonType() != ButtonType.AttackConfirm)
                    {
                        button.setPixelCoordinates((int)(pixelCoordinates.X + 32), (int)(pixelCoordinates.Y + (increment * 32)));
                        increment++;
                    }
                    else if (GameState.attackConfirmOpen)
                    {
                        button.setPixelCoordinates((int)(pixelCoordinates.X + 32), (int)(pixelCoordinates.Y));
                    }
                }
            }
        }

        /**
        returns the current sprite frame in animation sequence
        */
        public Rectangle getCurrentFrame() //return current frame the sprite is on
        {
            Rectangle screenBounds = new Rectangle(currentFrame * 32, 0, 32, 32);
            return screenBounds;
        }

        /**
        animate sprite walking the direction specified
        * @param direction The direction the unit is moving in 
        */
        public void animate(Direction direction)
        {

            #region Walking Down
            if (direction == Direction.Down) // for going down
            {
                if (currentFrame > 2) // if unit isnt already going down, set the sprite to default facing downwards
                {
                    currentFrame = 1;
                }
                else
                {
                    if (currentFrame == 2) // if unit has reached end of going down walk cycle, go back to beginning
                    {
                        currentFrame = 0;
                    }
                    else
                    {
                        currentFrame++; // increment through walk cycle
                    }
                }
            }
            #endregion

            #region Walking Left
            else if (direction == Direction.Left) // for going left
            {
                if ((currentFrame < 3) || (currentFrame > 5)) // if unit isnt already going left, set the sprite to default facing left
                {
                    currentFrame = 4;
                }
                else
                {
                    if (currentFrame == 5) // if unit has reached end of going down walk cycle, go back to beginning
                    {
                        currentFrame = 3;
                    }
                    else
                    {
                        currentFrame++; //increment walk cycle
                    }
                }
            }
            #endregion

            #region Walking Right
            else if (direction == Direction.Right) // for going right
            {
                if ((currentFrame < 6) || (currentFrame > 8)) // if unit isnt already going right, set the sprite to default facing right
                {
                    currentFrame = 7;//increment walk cycle
                }
                else
                {
                    if (currentFrame == 8)// if unit has reached end of going down walk cycle, go back to beginning
                    {
                        currentFrame = 6;
                    }
                    else
                    {
                        currentFrame++;
                    }
                }
            }
            #endregion

            #region Walking Up
            else if (direction == Direction.Up) // for going up
            {
                if (currentFrame < 9) // if unit isnt already going up, set the sprite to default facing upwards
                {
                    currentFrame = 10;//increment walk cycle
                }
                else
                {
                    if (currentFrame == 11)// if unit has reached end of going down walk cycle, go back to beginning
                    {
                        currentFrame = 9;//increment walk cycle
                    }
                    else
                    {
                        currentFrame++;
                    }
                }
            }
            #endregion
        }
    }
}
